@@ -1,4 +1,4 @@
-# Copyright 2020 Michael Thies <mail@mhthies.de>
+# Copyright 2020-2021 Michael Thies <mail@mhthies.de>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -14,7 +14,7 @@ import asyncio
 import contextvars
 import functools
 import logging
-from typing import Generic, List, Any, Tuple, Callable, Optional, Type, TypeVar, Awaitable, Union, Dict
+from typing import Generic, List, Any, Tuple, Callable, Optional, Type, TypeVar, Awaitable, Union, Dict, Set
 
 from . import conversion
 
@@ -37,12 +37,39 @@ class Connectable(Generic[T], metaclass=abc.ABCMeta):
     """
     type: Type[T]
 
-    def connect(self: C, other: "Connectable",
+    def connect(self: C, other: "Connectable[S]",
                 send: Optional[bool] = None,
                 receive: Optional[bool] = None,
                 read: Optional[bool] = None,
                 provide: Optional[bool] = None,
-                convert: Union[bool, Tuple[Callable[[T], Any], Callable[[Any], T]]] = False) -> C:
+                convert: Union[bool, Tuple[Callable[[T], S], Callable[[S], T]]] = False) -> C:
+        """
+        Subscribe self to other and set as default_provider and vice versa (depending on the two object's capabilities,
+        optionalities and given parameters).
+
+        :param other: The other *Connectable* object to connect with
+        :param send: Send value updates to `other` (i.e. subscribe `other` to `self`).
+            Requires `self` to be :class:`Subscribable` and other to be :class:`Writable`. In this case, defaults to
+            True if not specified.
+        :param receive: Receive value updates from `other` (i.e. subscribe `self` to `other`).
+            Requires `self` to be :class:`Writable` and other to be :class:`Subscribable`. In this case, defaults to
+            True if not specified.
+        :param read: Read values from `other` (i.e. set `other` as default provider for `self`).
+            Requires `self` to be :class:`Reading` and `other` to be :class:`Readable`.
+            If not specified, defaults to False unless `self.is_reading_optional` is set to False.
+        :param provide: Provide values to `other` for reading (i.e. set `self` as default provider for `other`).
+            Requires `self` to be :class:`Readable` and `other` to be :class:`Reading`.
+            If not specified, defaults to False unless `other.is_reading_optional` is set to False.
+        :param convert: Enable built-in type conversion for the created subscriptions/default_providers:
+            Either a boolean or a tuple of two conversion functions. Defaults to ``False``, i.e. a type mismatch will
+            result in a TypeError instead of implicit conversion. Set to ``True``, to choose the appropriate
+            conversion function automatically. Raises a TypeError if no default conversion for one of the two directions
+            is available and that direction is used.
+            For custom conversion functions, pass a tuple ``(to_other, to_self)``, where ``to_other`` is a (non-async)
+            callable for converting `self`'s value type into `other`'s value type and ``to_self`` does the conversion
+            the other way round.
+        :return: Returns self to allow functional-style chaining
+        """
         if isinstance(other, ConnectableWrapper):
             # If other object is not connectable itself but wraps one or more connectable objects (like, for example, a
             # `web.widgets.ValueButtonGroup`), let it use its special implementation of `connect()`.
@@ -146,6 +173,11 @@ class Writable(Connectable[T_con], Generic[T_con], metaclass=abc.ABCMeta):
 class Readable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
     @abc.abstractmethod
     async def read(self) -> T_co:
+        """
+        Get the current value of this Connectable object.
+
+        :raises UninitializedError: If the value is not (yet) specified
+        """
         pass
 
 
@@ -153,14 +185,17 @@ class UninitializedError(RuntimeError):
     pass
 
 
+SubscriberListType = Tuple[Writable[S], Optional[Callable[[T_co], S]]]
+
+
 class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
     _stateful_publishing: bool = False
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._subscribers: List[Tuple[Writable[S], Optional[Callable[[T_co], S]]]] = []
+        self._subscribers: List[SubscriberListType] = []
         self._triggers: List[Tuple[LogicHandler, bool]] = []
-        self._pending_updates: Dict[int, Set[asyncio.Task]] = {}
+        self._pending_updates: Dict[int, Dict[asyncio.Task, Optional[int]]] = {}
 
     async def __publish_write(self, subscriber: Writable[S], converter: Optional[Callable[[T_co], S]], value: T_co,
                               origin: List[Any], use_pending: bool):
@@ -170,7 +205,7 @@ class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
             logger.error("Error while writing new value %s from %s to %s:", value, self, subscriber, exc_info=e)
         finally:
             if use_pending:
-                self._pending_updates[id(subscriber)].discard(asyncio.current_task())
+                del self._pending_updates[id(subscriber)][asyncio.current_task()]  # type: ignore
 
     async def __publish_trigger(self, target: LogicHandler, value: T_co,
                                 origin: List[Any], use_pending: bool):
@@ -180,7 +215,7 @@ class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
             logger.error("Error while triggering %s from %s:", target, self, exc_info=e)
         finally:
             if use_pending:
-                self._pending_updates[id(target)].discard(asyncio.current_task())
+                del self._pending_updates[id(target)][asyncio.current_task()]  # type: ignore
 
     def _publish(self, value: T_co, origin: List[Any]):
         """
@@ -206,20 +241,22 @@ class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
         """
         if self._stateful_publishing:
             for subscriber, converter in self._subscribers:
-                reset_origin = bool(self._pending_updates[id(subscriber)])
+                prev_step = id(origin[-1]) if origin else None
+                reset_origin = any(o != prev_step for o in self._pending_updates[id(subscriber)].values())
                 if reset_origin:
                     logger.info("Resetting origin from %s to %s; value=%s; origin=%s", self, subscriber, value, origin)
                 if reset_origin or not any(s is subscriber for s in origin):
                     task = asyncio.create_task(self.__publish_write(subscriber, converter, value,
                                                                     [] if reset_origin else origin, True))
-                    self._pending_updates[id(subscriber)].add(task)
+                    self._pending_updates[id(subscriber)][task] = prev_step
             for target, sync in self._triggers:
                 reset_origin = False
                 if sync:
-                    reset_origin = bool(self._pending_updates[id(target)])
+                    prev_step = id(origin[-1]) if origin else None
+                    reset_origin = any(o != prev_step for o in self._pending_updates[id(target)].values())
                 task = asyncio.create_task(self.__publish_trigger(target, value, [] if reset_origin else origin, sync))
                 if sync:
-                    self._pending_updates[id(target)].add(task)
+                    self._pending_updates[id(target)][task] = prev_step
 
         else:
             for target, sync in self._triggers:
@@ -285,7 +322,7 @@ class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
                             .format(repr(subscriber), subscriber.type.__name__, repr(self), self.type.__name__))
         self._subscribers.append((subscriber, converter))
         if self._stateful_publishing:
-            self._pending_updates[id(subscriber)] = set()
+            self._pending_updates[id(subscriber)] = {}
 
     def trigger(self, target: LogicHandler, synchronous: bool = False) -> LogicHandler:
         """
@@ -323,20 +360,27 @@ class Subscribable(Connectable[T_co], Generic[T_co], metaclass=abc.ABCMeta):
 
         :param target: The handler function/coroutine to be triggered on updates. Must comply with the requirements
             mentioned above.
-        :return: The ``target`` function (unchanged)
+        :param synchronous: If True, the target coroutine is triggered synchronously, i.e. :meth:`_publish_and_wait`
+            will wait for it to return. This SHALL be used for internal triggering methods that re-publish the value
+            updates of this Subscribable object. For logic handlers that might require substantial time to return, it
+            SHALL not be used!
+        :return: The ``target`` function (unchanged) – this allows decorator-style usage of this method
         """
         self._triggers.append((target, synchronous))
         if synchronous and self._stateful_publishing:
-            self._pending_updates[id(target)] = set()
+            self._pending_updates[id(target)] = {}
         return target
 
 
+DefaultProviderType = Tuple[Readable[S], Optional[Callable[[S], T_con]]]
+
+
 class Reading(Connectable[T_con], Generic[T_con], metaclass=abc.ABCMeta):
     is_reading_optional: bool = True
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._default_provider: Optional[Tuple[Readable[S], Optional[Callable[[S], T_con]]]] = None
+        self._default_provider: Optional[DefaultProviderType] = None
 
     def set_provider(self, provider: Readable[S], convert: Union[Callable[[S], T_con], bool] = False):
         converter: Optional[Callable[[S], T_con]]
