@@ -18,7 +18,7 @@ from typing import Type, Dict, Generic, List, Any, Optional
 import aiohttp
 
 from ._helper import SupervisedClientInterface
-from ..base import T, Subscribable, Writable, Readable, UninitializedError
+from ..base import T, Subscribable, Writable, Readable, UninitializedError, Reading
 from ..conversion import SHCJsonEncoder, from_json
 from ..supervisor import register_interface, stop
 
@@ -47,10 +47,16 @@ class SHCWebClient(SupervisedClientInterface):
         connection can not be established in the first try. The connection is retried in background with exponential
         backoff (see `auto_reconnect` option). Otherwise (default), the first connection attempt on startup is not
         retried and will shutdown the SHC application on failure, even if `auto_reconnect` is True.
+    :param client_online_object: The (optional) name of an API object of the SHC API server to be used as a server-side
+        online indicator for this client. The object must be of bool type. The client will automatically send a `True`
+        value to this object when (re)connecting and use the server's ‘last will’ feature to let a `False` value be
+        published when the websocket connection is lost.
     """
-    def __init__(self, server: str, auto_reconnect: bool = True, failsafe_start: bool = False) -> None:
+    def __init__(self, server: str, auto_reconnect: bool = True, failsafe_start: bool = False,
+                 client_online_object: Optional[str] = None) -> None:
         super().__init__(auto_reconnect, failsafe_start)
         self.server = server
+        self.client_online_object = client_online_object
         self._api_objects: Dict[str, WebApiClientObject] = {}
 
         self._session: aiohttp.ClientSession
@@ -88,10 +94,16 @@ class SHCWebClient(SupervisedClientInterface):
         self._ws = await self._session.ws_connect(self.server + '/api/v1/ws')
 
     async def _subscribe(self) -> None:
+        # First send the value all objects with a default_provider to the server
+        await asyncio.gather(*(obj._read_and_send()
+                               for obj in self._api_objects.values()))
+        # then subscribe all objects with subscribers for updates from the server
         await asyncio.gather(*(self._subscribe_and_wait(name)
                                for name, obj in self._api_objects.items()
                                if obj._subscribers or obj._triggers))
-        # TODO gather results and give a better error description
+        if self.client_online_object is not None:
+            await self._set_last_will(self.client_online_object, False)
+            await self._send_value(self.client_online_object, True)
 
     async def _disconnect(self) -> None:
         logger.info("Closing client websocket to %s ...", self.server)
@@ -183,6 +195,37 @@ class SHCWebClient(SupervisedClientInterface):
             raise WebSocketAPIError("Writing value to SHC API failed with error {}: {}"
                                     .format(result['status'], result.get('error')))
 
+    async def _set_last_will(self, name: str, value: Any) -> None:
+        """
+        Coroutine for setting a last will at the SHC server
+
+        The method awaits the receipt of the server's answer or a timeout of TIMEOUT seconds. In case of a server side
+        error or a response timeout, an exception is raised.
+
+        The parameters are similar to :meth:`_send_value`:
+
+        :param name: Name of the API object to set the last will for
+        :param value: The last will value to be stored at the server. The value's type must match the server-side API
+            object's type and be encodable with the SHCJsonEncoder
+        :raises WebSocketAPIError: when setting the last will fails on the server side (i.e. the API's return code is
+            not in the 200-range). This may be caused by a type or object name mismatch.
+        :raises asyncio.TimeoutError: when no response is received from the server within TIMEOUT seconds
+        """
+        if self._ws is None:
+            raise RuntimeError("Websocket of SHC client for API at {} has not been connected yet".format(self.server))
+        future = asyncio.get_event_loop().create_future()
+        self._waiting_futures[id(future)] = future
+        logger.debug("Setting last will at SHC API for object name %s ...", name)
+        await self._ws.send_str(json.dumps({'action': 'lastwill', 'name': name, 'value': value, 'handle': id(future)},
+                                           cls=SHCJsonEncoder))
+
+        result = await asyncio.wait_for(future, TIMEOUT)
+        if 200 <= result['status'] < 300:
+            logger.debug("Setting last will at SHC API for object %s succeeded", name)
+        else:
+            raise WebSocketAPIError("Setting last will at SHC API failed with error {}: {}"
+                                    .format(result['status'], result.get('error')))
+
     async def _read_value(self, name: str) -> Any:
         """
         Coroutine called by WebApiClientObject's read() to fetch the current value of an API object from the remote SHC
@@ -222,8 +265,8 @@ class SHCWebClient(SupervisedClientInterface):
         Create a `Connectable` object for communicating with an API object on the remote SHC server.
 
         The returned object is `Subscribable`, `Readable` and `Writable`. Read and write calls are basically forwarded
-        to the corresponding :class:`shc.web.WebApiObject`; new values, pushed from the server, are published to the
-        object's subscribers.
+        to the corresponding :class:`WebApiObject <shc.web.interface.WebApiObject>` of the server; new values, pushed
+        from the server, are published to the object's subscribers.
 
         :param type_: The data type of the server's API object. This is also the `type` of the returned `Connectable`
             object.
@@ -244,7 +287,7 @@ class SHCWebClient(SupervisedClientInterface):
         return "{}(server={})".format(self.__class__.__name__, self.server)
 
 
-class WebApiClientObject(Readable[T], Writable[T], Subscribable[T], Generic[T]):
+class WebApiClientObject(Readable[T], Writable[T], Subscribable[T], Reading[T], Generic[T]):
     """
     A `Connectable` object to communicate with a single API object of a remote SHC websocket API.
 
@@ -254,23 +297,47 @@ class WebApiClientObject(Readable[T], Writable[T], Subscribable[T], Generic[T]):
 
     Thus, this class inherits from :class:`shc.base.Readable`, :class:`shc.base.Wrtiable` and
     :class:`shc.base.Subscribable`.
+
+    In addition, the class inherits from :class:`shc.base.Reading` (but with `is_reading_optional = True`). This is used
+    to (optionally) read a local provider's value and send it to the server upon (re)connection. This is done **before**
+    subscribing for updates from the server to ensure that the local value prevails over the server's value in case of
+    diverged values in the unconnected time.
     """
     _stateful_publishing = True
+    is_reading_optional = True
 
     def __init__(self, client: SHCWebClient, type_: Type[T], name: str):
         self.type = type_
         super().__init__()
         self.client = client
         self.name = name
+        self.pending_sends = 0
 
     async def read(self) -> T:
         return from_json(self.type, await self.client._read_value(self.name))
 
-    async def _write(self, value: T, origin: List[Any]) -> None:
+    async def _write(self, value: T, origin: List[Any], _reflected_from_server: bool = False) -> None:
         # Asynchronous local feedback publishing. This ensures that conflicting updates, which are currently waiting for
         # local processing by a subscriber can be corrected by resetting this update's origin.
         self._publish(value, origin)
-        await self.client._send_value(self.name, value)
+        if not _reflected_from_server:
+            self.pending_sends += 1
+        try:
+            await self.client._send_value(self.name, value)
+        finally:
+            if not _reflected_from_server:
+                self.pending_sends -= 1
+
+    async def _read_and_send(self) -> None:
+        """
+        If a default_provider is set, read its current value and send it to the server.
+
+        This is usually called by the :meth:`SHCWebClient._subscribe` to send the current local value to the server upon
+        (re)connect.
+        """
+        value = await self._from_provider()
+        if value is not None:
+            await self._write(value, [])
 
     def new_value(self, value: Any) -> None:
         """
@@ -280,6 +347,10 @@ class WebApiClientObject(Readable[T], Writable[T], Subscribable[T], Generic[T]):
         :param value: The received, json-decoded value. It will be processed using :meth:`shc.conversion.from_json`.
         """
         try:
+            # Return value to server if we have sent a (conflicting) value recently (with the acknowledge still pending)
+            # to prevent inconsistent state on server and client.
+            if self.pending_sends:
+                asyncio.create_task(self._write(value, [], _reflected_from_server=True))
             self._publish(from_json(self.type, value), [])
             logger.debug("Received new value %s for SHC API object %s", value, self.name)
         except Exception as e:
