@@ -1,4 +1,4 @@
-# Copyright 2020 Michael Thies <mail@mhthies.de>
+# Copyright 2020-2021 Michael Thies <mail@mhthies.de>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -25,7 +25,7 @@ _REGISTERED_INTERFACES: Set["AbstractInterface"] = set()
 
 event_loop = asyncio.get_event_loop()
 _EXIT_CODE = 0
-_SHC_STOPPED = asyncio.Event(loop=event_loop)
+_SHC_STOPPED = asyncio.Event()
 _SHC_STOPPED.set()
 
 
