@@ -1,4 +1,4 @@
-# Copyright 2020 Michael Thies <mail@mhthies.de>
+# Copyright 2020-2021 Michael Thies <mail@mhthies.de>
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 # the License. You may obtain a copy of the License at
@@ -39,6 +39,8 @@ jinja_env = jinja2.Environment(
 )
 jinja_env.filters['id'] = id
 
+LastWillT = Tuple["WebApiObject[T]", T]
+
 
 class WebServer(AbstractInterface):
     """
@@ -79,6 +81,8 @@ class WebServer(AbstractInterface):
         self._websockets: weakref.WeakSet[aiohttp.web.WebSocketResponse] = weakref.WeakSet()
         # a set of all open tasks to close on graceful shutdown
         self._associated_tasks: weakref.WeakSet[asyncio.Task] = weakref.WeakSet()
+        # last will (object, value) per API websocket client (if set)
+        self._api_ws_last_will: Dict[aiohttp.web.WebSocketResponse, LastWillT] = {}
         # data structure of the user interface's main menu
         # The structure looks as follows:
         # [('Label', 'icon', 'page_name'),
@@ -89,11 +93,11 @@ class WebServer(AbstractInterface):
         self.ui_menu_entries: List[Tuple[str, Optional[str], Union[str, List[Tuple[str, Optional[str], str]]]]] = []
         # List of all static js URLs to be included in the user interface pages
         self._js_files = [
-            "static/pack/main.js",
+            "/static/pack/main.js",
         ]
         # List of all static css URLs to be included in the user interface pages
         self._css_files = [
-            "static/pack/main.css",
+            "/static/pack/main.css",
         ]
         # A dict of all static files served by the application. Used to make sure, any of those is only served at one
         # path, when added via `serve_static_file()` multiple times.
@@ -303,6 +307,9 @@ class WebServer(AbstractInterface):
             self._websockets.discard(ws)
             for api_object in self._api_objects.values():
                 api_object.websocket_close(ws)
+            if ws in self._api_ws_last_will:
+                obj, value = self._api_ws_last_will.pop(ws)
+                await obj.http_post(value, ws)
             return ws
 
     async def _api_websocket_dispatch(self, request: aiohttp.web.Request, ws: aiohttp.web.WebSocketResponse,
@@ -366,6 +373,28 @@ class WebServer(AbstractInterface):
                         result['status'] = 422
                         result['error'] = "Could not use provided value to update API object: {}".format(e)
 
+            # lastwill action
+            elif action == "lastwill":
+                value_exists = False
+                try:
+                    value = message["value"]
+                    value_exists = True
+                except KeyError:
+                    result['status'] = 422
+                    result['error'] = "message does not include a 'value' field"
+                    logger.warning("Websocket API LASTWILL message from %s without 'value' field: %s", request.remote,
+                                   message)
+                if value_exists:
+                    logger.debug("got LASTWILL request for API object %s via websocket from %s with value %s",
+                                 name, request.remote, value)
+                    try:
+                        self._api_ws_last_will[ws] = (obj, obj._check_last_will(value))
+                    except (ValueError, TypeError) as e:
+                        logger.warning("Error while setting last will of websocket client %s for API object %s (error"
+                                       "was %s): %s", name, request.remote, e, value)
+                        result['status'] = 422
+                        result['error'] = "Could not use provided value to set last will: {}".format(e)
+
             # get action
             elif action == "get":
                 logger.debug("got get request for API object %s via websocket from %s", name, request.remote)
@@ -513,56 +542,69 @@ class WebServer(AbstractInterface):
         creating a new one.
 
         This method should primarily be used by WebPageItem implementations within their
-        :meth:`WebPageItem.register_with_server` method.
+        :meth:`WebPageItem.register_with_server` method. It is meant for serving configuration-specific images etc.
 
         :param path: The path of the local file to be served as a static file
-        :return: The URL of the static file, as a path, relative to the server's root URL, with leading slash. For
-            using it within the web UI's HTML code, the server's `root_url` must be prepended.
+        :return: The URL of the static file, including the server's root_url, such that it is represented an absolute
+            URL or absolute-path reference (relative to the HTTP server root), which can be used in <img>, <link> tags,
+            etc.
         """
         path = path.absolute()
         if path in self.static_files:
-            return self.static_files[path]
+            return f"{self.root_url}/addon/{self.static_files[path]}"
+
         final_file_name = path.name
         i = 0
-        while final_file_name in self.static_files:
+        while final_file_name in self.static_files.values():
             final_file_name = "{}_{:04d}.{}".format(path.stem, i, path.suffix)
-        final_path = '/addon/{}'.format(final_file_name)
-        self.static_files[path] = final_path
+        self.static_files[path] = final_file_name
+        final_url = '/addon/{}'.format(final_file_name)
 
         # Unfortunately, aiohttp.web.static can only serve directories. We want to serve a single file here.
         async def send_file(_request):
             return aiohttp.web.FileResponse(path)
-        self._app.add_routes([aiohttp.web.get(final_path, send_file)])
+        self._app.add_routes([aiohttp.web.get(final_url, send_file)])
 
-        return final_path
+        return self.root_url + final_url
 
-    def add_js_file(self, path: pathlib.Path) -> None:
+    def add_static_directory(self, path: pathlib.Path, js_files: Iterable[str] = (), css_files: Iterable[str] = ()
+                             ) -> str:
         """
-        Register an additional static JavaScript file to be included in the web UI served by this server.
+        Register an additional directory of static files served by this server, optionally with a list of JavaScript and
+        CSS files to be loaded in each page's HTML head.
 
-        This method adds the given path as a static file to the webserver and includes its URL into every web UI page
-        using a `<script>` tag in the HTML head.
-        If the same file has already been added as a static file to the webserver, this method does nothing.
+        This method adds the given path as a static directory to be served by this webserver. The root URL of the served
+        directory is automatically determined, based on the file's name and existing static files. If the same path has
+        already been added as a static file, its existing static URL is returned instead of creating a new one.
 
-        :param path: Local filesystem path of the JavaScript file to be included
+        The given `js_files` and `css_files` are interpreted as relative URL references of files within the directory,
+        that will be served by the HTTP server at the directory's root URL + this path. Make sure to use '/' as path
+        separator and URL-encode these strings if necessary.
+
+        :param path: Local filesystem path of directory to be served
+        :param js_files: list of relative URLs within the directory to be included as Javascript files into UI pages
+        :param css_files: list of relative URLs within the directory to be included as CSS files into UI pages
+        :return: The HTTP root URL where the directory is served, including the server's root_url, such that it is
+            represented an absolute URL or absolute-path reference (relative to the HTTP server root)
         """
+        path = path.absolute()
         if path in self.static_files:
-            return
-        self._js_files.append(self.serve_static_file(path))
+            final_url = '/addon/{}'.format(self.static_files[path])
+        else:
+            final_file_name = path.name
+            i = 0
+            while final_file_name in self.static_files.values():
+                final_file_name = "{}_{:04d}.{}".format(path.stem, i, path.suffix)
 
-    def add_css_file(self, path: pathlib.Path) -> None:
-        """
-        Register an additional static CSS file to be included in the web UI served by this server.
+            self.static_files[path] = final_file_name
+            final_url = '/addon/{}'.format(final_file_name)
 
-        This method adds the given path as a static file to the webserver and includes its URL into every web UI page
-        using a `<link rel="stylesheet">` tag in the HTML head.
-        If the same file has already been added as a static file to the webserver, this method does nothing.
+            self._app.add_routes([aiohttp.web.static(final_url, path)])
 
-        :param path: Local filesystem path of the CSS file to be included
-        """
-        if path in self.static_files:
-            return
-        self._css_files.append(self.serve_static_file(path))
+        self._css_files.extend((f"{final_url}/{file}" for file in css_files))
+        self._js_files.extend((f"{final_url}/{file}" for file in js_files))
+
+        return f"{self.root_url}{final_url}/"
 
     def __repr__(self) -> str:
         return "{}(host={}, port={})".format(self.__class__.__name__, self.host, self.port)
@@ -855,6 +897,24 @@ class WebApiObject(Reading[T], Writable[T], Subscribable[T], Generic[T]):
         self._publish(from_json(self.type, value), [origin])
         await self._publish_http(value, origin)
 
+    def _check_last_will(self, value: Any) -> T:
+        """
+        Check and convert a value that has been send by an API client as a last will for this object.
+
+        We do this synchronously when the client set its last will, so we can inform it if the value is invalid.
+
+        :param value: The decoded json value from the client
+        :return: The converted value to be stored for later use as a last will
+        """
+        return from_json(self.type, value)
+
+    async def _post_last_will(self, value: T, ws: aiohttp.web.WebSocketResponse) -> None:
+        """
+        Post the stored last will of a client
+        """
+        self._publish(from_json(self.type, value), [ws])
+        await self._publish_http(value, ws)
+
     async def _publish_http(self, value: T, skip_websocket: Optional[object] = None) -> None:
         """
         Publish a new value to all subscribed websockets and waiting long-running poll requests.
@@ -918,7 +978,7 @@ class WebApiObject(Reading[T], Writable[T], Subscribable[T], Generic[T]):
 
         # If waiting for next value: Await future using timeout
         try:
-            value = await asyncio.wait_for(self.future, timeout=timeout)
+            value = await asyncio.wait_for(asyncio.shield(self.future), timeout=timeout)
             return True, value, str(id(self.future))
         except (asyncio.TimeoutError, asyncio.CancelledError):
             return False, None, str(id(self.future))
