@@ -37,7 +37,7 @@ from markupsafe import Markup
 
 from .interface import WebPageItem, WebDisplayDatapoint, WebActionDatapoint, jinja_env, WebConnectorContainer,\
     WebUIConnector, WebPage, WebServer
-from ..base import T, ConnectableWrapper
+from ..base import T, ConnectableWrapper, Connectable
 from ..conversion import SHCJsonEncoder
 from ..datatypes import RangeFloat1, RGBUInt8
 
@@ -506,6 +506,7 @@ class ValueListButtonGroup(ButtonGroup, ConnectableWrapper[T], Generic[T]):
 
     def connect(self, *args, **kwargs):
         for button in self.buttons:
+            assert isinstance(button, Connectable)
             button.connect(*args, **kwargs)
         return self
 
@@ -587,7 +588,7 @@ class ImageMap(WebPageItem):
 
     def register_with_server(self, _page: WebPage, server: WebServer) -> None:
         if not self.image_url:
-            self.image_url = server.root_url + server.serve_static_file(pathlib.Path(self.image))
+            self.image_url = server.serve_static_file(pathlib.Path(self.image))
 
     def get_connectors(self) -> Iterable[WebUIConnector]:
         for x, y, item, sub_items in self.items:
