@@ -1,3 +1,13 @@
+# Copyright 2020-2021 Michael Thies <mail@mhthies.de>
+#
+# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an
+# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
+# specific language governing permissions and limitations under the License.
 import asyncio
 import concurrent.futures
 import functools
@@ -6,7 +16,7 @@ import threading
 import time
 import unittest.mock
 import datetime
-from typing import Callable, Any, Awaitable, TypeVar, Generic, Type, List, Union, Tuple, Set, Optional
+from typing import Callable, Any, Awaitable, TypeVar, Generic, Type, List, Union, Tuple, Set, Optional, Coroutine
 
 from shc import base
 
@@ -181,15 +191,15 @@ class ClockMock:
             unittest.mock.patch('datetime.date.today', new=self.today),
         )
         for p in self.patches:
-            p.__enter__()
+            p.__enter__()  # type: ignore  # MyPy does not seem to get the _patch objects right
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         for p in self.patches:
-            p.__exit__(exc_type, exc_val, exc_tb)
+            p.__exit__(exc_type, exc_val, exc_tb)  # type: ignore
 
-        datetime.datetime = self.original_datetime
-        datetime.date = self.original_date
+        datetime.datetime = self.original_datetime  # type: ignore
+        datetime.date = self.original_date  # type: ignore
 
 
 # ###########################
@@ -257,11 +267,14 @@ class InterfaceThreadRunner(Generic[IT]):
 
     The interface will most probably not be thread-safe internally (as SHC usually runs in only one AsyncIO event loop
     in a single thread). Thus, its methods/coroutines must only be called within the `InterfaceThreadRunner`s AsnycIO
-    event loop. For convenience, there is a :meth:`start` method that calls the interface's start method correctly::
+    event loop. For convenience, there is a :meth:`start` method that calls the interface's start method correctly, as
+    well as two methods :meth:`run_coro` and :meth:`run_coro_async` to run any coroutine in the context of the event
+    loop and return its result (or raise any exceptions):
 
         runner = InterfaceThreadRunner(SomeSHCInterfaceClass, "arg1", "arg2")
         runner.start()
-        asyncio.run_coroutine_threadsafe(interface.some_coro(*args), loop=runner.loop).result()
+        runner.run_coro(interface.some_coro(*args))
+        # or: await runner.run_coro_async(interface.some_coro(*args))
         runner.loop.call_soon_threadsafe(interface.some_method(*args))
         runner.stop()
 
@@ -288,7 +301,7 @@ class InterfaceThreadRunner(Generic[IT]):
         self.loop = asyncio.new_event_loop()
         asyncio.set_event_loop(self.loop)
         try:
-            self.interface = interface_class(*args, **kwargs)  # type: ignore
+            self.interface = interface_class(*args, **kwargs)
             self._server_constructed_future.set_result(None)
         except Exception as e:
             self._server_constructed_future.set_exception(e)
@@ -313,3 +326,9 @@ class InterfaceThreadRunner(Generic[IT]):
         stop_future.result(timeout=5)
         self.loop.call_soon_threadsafe(self._stopped_event.set)
         self.future.result(timeout=5)
+
+    def run_coro(self, coroutine: Coroutine[None, None, T]) -> T:
+        return asyncio.run_coroutine_threadsafe(coroutine, loop=self.loop).result()
+
+    async def run_coro_async(self, coroutine: Coroutine[None, None, T]) -> T:
+        return await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(coroutine, loop=self.loop))
