@@ -6,10 +6,11 @@ import time
 import unittest
 import unittest.mock
 from contextlib import suppress
+from pathlib import Path
 from typing import List
 
-import asyncio_mqtt  # type: ignore
-from paho.mqtt.client import MQTTMessage  # type: ignore
+import asyncio_mqtt
+from paho.mqtt.client import MQTTMessage
 
 import shc.interfaces.mqtt
 
@@ -21,7 +22,8 @@ class MQTTClientTest(unittest.TestCase):
     def setUp(self) -> None:
         self.client_runner = InterfaceThreadRunner(shc.interfaces.mqtt.MQTTClientInterface, "localhost", 42883)
         self.client = self.client_runner.interface
-        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883"])
+        self.broker_config_file = Path(__file__).parent.parent / 'assets' / 'mosquitto.conf'
+        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883", '-c', str(self.broker_config_file)])
         time.sleep(0.25)
 
     def tearDown(self) -> None:
@@ -99,16 +101,14 @@ class MQTTClientTest(unittest.TestCase):
 
             self.client_runner.start()
 
-            await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(conn_raw.write(b'"500"', [self]),
-                                                                       loop=self.client_runner.loop))
+            await self.client_runner.run_coro_async(conn_raw.write(b'"500"', [self]))
             # Write might not wait until the message is published
             await asyncio.sleep(0.1)
             self.assertEqual(1, len(MESSAGES))
             self.assertEqual(b'"500"', MESSAGES[-1].payload)
             self.assertEqual('test/topic', MESSAGES[-1].topic)
 
-            await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(conn_str.write('a test with »«', [self]),
-                                                                       loop=self.client_runner.loop))
+            await self.client_runner.run_coro_async(conn_str.write('a test with »«', [self]))
             # Due to the local subscriber, write() should wait until the message has been received
             target_str._write.assert_called_once_with('a test with »«', [self, conn_str])
             await asyncio.sleep(0.01)
@@ -116,8 +116,7 @@ class MQTTClientTest(unittest.TestCase):
             self.assertEqual('a test with »«'.encode('utf-8'), MESSAGES[-1].payload)
             self.assertEqual('test/another/topic', MESSAGES[-1].topic)
 
-            await asyncio.wrap_future(asyncio.run_coroutine_threadsafe(conn_json.write('text', [self]),
-                                                                       loop=self.client_runner.loop))
+            await self.client_runner.run_coro_async(conn_json.write('text', [self]))
             # Du to the forced subscription, write() should wait until the message has been received
             await asyncio.sleep(0.01)
             self.assertEqual(3, len(MESSAGES))
@@ -143,17 +142,17 @@ class MQTTClientTest(unittest.TestCase):
             self.broker_process.terminate()
             self.broker_process.wait()
             time.sleep(0.2)
-        self.assertIn("Disconnected", ctx.output[0])
-        self.assertIn("MQTTClientInterface", ctx.output[0])
+        self.assertIn("Disconnected", ctx.output[0])  # type: ignore
+        self.assertIn("MQTTClientInterface", ctx.output[0])  # type: ignore
 
         # Wait for first reconnect attempt
         with self.assertLogs("shc.interfaces._helper", logging.ERROR) as ctx:
             time.sleep(1.1)
-        self.assertIn("Error in interface MQTTClientInterface", ctx.output[0])
-        self.assertIn("Connection refused", ctx.output[0])
+        self.assertIn("Error in interface MQTTClientInterface", ctx.output[0])  # type: ignore
+        self.assertIn("Connection refused", ctx.output[0])  # type: ignore
 
         # Restart server
-        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883"])
+        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883", '-c', str(self.broker_config_file)])
 
         # Wait for second reconnect attempt
         with unittest.mock.patch.object(self.client.client, 'connect', new=AsyncMock()) as connect_mock:
@@ -178,11 +177,11 @@ class MQTTClientTest(unittest.TestCase):
         with self.assertLogs("shc.interfaces._helper", logging.ERROR) as ctx:
             self.client_runner.start()
             time.sleep(0.5)
-        self.assertIn("Error in interface MQTTClientInterface", ctx.output[0])
-        self.assertIn("Connection refused", ctx.output[0])
+        self.assertIn("Error in interface MQTTClientInterface", ctx.output[0])  # type: ignore
+        self.assertIn("Connection refused", ctx.output[0])  # type: ignore
 
         # Restart server
-        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883"])
+        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883", '-c', str(self.broker_config_file)])
         time.sleep(0.25)
         asyncio.get_event_loop().run_until_complete(self._send_retained_test_message())
 
