@@ -6,9 +6,10 @@ import time
 import unittest
 import unittest.mock
 from contextlib import suppress
+from pathlib import Path
 from typing import Dict, Any
 
-import asyncio_mqtt  # type: ignore
+import asyncio_mqtt
 
 import shc.interfaces.mqtt
 import shc.interfaces.tasmota
@@ -25,13 +26,13 @@ class TasmotaInterfaceTest(unittest.TestCase):
         self.client_runner = InterfaceThreadRunner(shc.interfaces.mqtt.MQTTClientInterface, "localhost", 42883)
         self.client = self.client_runner.interface
         self.interface: shc.interfaces.tasmota.TasmotaInterface = \
-            asyncio.run_coroutine_threadsafe(_construct(self.client, 'test-device'), loop=self.client_runner.loop)\
-            .result()
-        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883"])
+            self.client_runner.run_coro(_construct(self.client, 'test-device'))
+        broker_config_file = Path(__file__).parent.parent / 'assets' / 'mosquitto.conf'
+        self.broker_process = subprocess.Popen(["mosquitto", "-p", "42883", '-c', str(broker_config_file)])
         time.sleep(0.25)
 
     def tearDown(self) -> None:
-        asyncio.run_coroutine_threadsafe(self.interface.stop(), loop=self.client_runner.loop).result()
+        self.client_runner.run_coro(self.interface.stop())
         self.client_runner.stop()
         self.broker_process.terminate()
         self.broker_process.wait()
@@ -44,7 +45,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
         try:
             target_offline = ExampleWritable(bool).connect(self.interface.online())
             self.client_runner.start()
-            asyncio.run_coroutine_threadsafe(self.interface.start(), loop=self.client_runner.loop).result()
+            self.client_runner.run_coro(self.interface.start())
             await asyncio.sleep(0.25)
 
             target_offline._write.assert_called_once_with(True, unittest.mock.ANY)
@@ -76,7 +77,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
 
         try:
             self.client_runner.start()
-            asyncio.run_coroutine_threadsafe(self.interface.start(), loop=self.client_runner.loop).result()
+            self.client_runner.run_coro(self.interface.start())
 
             await asyncio.sleep(0.25)
             target_color._write.assert_called_once_with(construct_color(0, 0, 0, 0), unittest.mock.ANY)
@@ -110,7 +111,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
 
         try:
             self.client_runner.start()
-            asyncio.run_coroutine_threadsafe(self.interface.start(), loop=self.client_runner.loop).result()
+            self.client_runner.run_coro(self.interface.start())
 
             await asyncio.sleep(0.25)
             target_color._write.assert_called_once_with(construct_color(0, 0, 0, 0), unittest.mock.ANY)
@@ -121,7 +122,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
             # Should only return when the message returns from the broker
             await asyncio.sleep(0.001)
             self.assertFalse(future.done())
-            await future
+            await asyncio.wait_for(future, timeout=1.0)
 
             target_color._write.assert_called_with(construct_color(175, 117, 48, 66), [self, conn_color])
             target_power._write.assert_called_with(True, [self, conn_power])
@@ -130,7 +131,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
         finally:
             task.cancel()
             with suppress(asyncio.CancelledError):
-                await task
+                await asyncio.wait_for(task, timeout=1.0)
 
     @async_test
     async def test_sensor_ir(self) -> None:
@@ -138,7 +139,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
         target_ir = ExampleWritable(bytes).connect(conn_ir)
 
         self.client_runner.start()
-        asyncio.run_coroutine_threadsafe(self.interface.start(), loop=self.client_runner.loop).result()
+        self.client_runner.run_coro(self.interface.start())
         await asyncio.sleep(0.25)
 
         target_ir._write.assert_not_called()
@@ -170,7 +171,7 @@ class TasmotaInterfaceTest(unittest.TestCase):
         target_total = ExampleWritable(float).connect(conn_total)
 
         self.client_runner.start()
-        asyncio.run_coroutine_threadsafe(self.interface.start(), loop=self.client_runner.loop).result()
+        self.client_runner.run_coro(self.interface.start())
         await asyncio.sleep(0.25)
 
         target_power._write.assert_not_called()
@@ -262,5 +263,6 @@ async def tasmota_device_mock(deviceid: str) -> None:
                                                     "Channel": [int(v/255*100) for v in channel]}).encode('ascii'))
 
         except asyncio.CancelledError:
+            print("Mock cancelled")
             await c.publish(f"tele/{deviceid}/LWT", b'Offline', retain=True)
             raise
