@@ -40,7 +40,7 @@ class TestSubscribe(unittest.TestCase):
         a.subscribe(b)
         with self.assertLogs(level=logging.ERROR) as ctx:
             await a.publish(TOTALLY_RANDOM_NUMBER, [self])
-        self.assertIn("unexpected error in _write", "\n".join(ctx.output))
+        self.assertIn("unexpected error in _write", "\n".join(ctx.output))  # type: ignore
 
     @async_test
     async def test_type_conversion(self) -> None:
@@ -78,7 +78,7 @@ class TestHandler(unittest.TestCase):
         a = ExampleSubscribable(int)
         handler = AsyncMock()
         handler.__name__ = "handler_mock"
-        wrapped_handler = base.handler()(handler)  # type: ignore
+        wrapped_handler = base.handler()(handler)
         a.trigger(wrapped_handler)
         await a.publish(TOTALLY_RANDOM_NUMBER, [self])
         await asyncio.sleep(0.01)
@@ -158,7 +158,7 @@ class TestHandler(unittest.TestCase):
         with self.assertLogs(level=logging.ERROR) as ctx:
             await a.publish(TOTALLY_RANDOM_NUMBER, [self])
             await asyncio.sleep(0.01)
-        self.assertIn("unexpected error in _write", "\n".join(ctx.output))
+        self.assertIn("unexpected error in _write", "\n".join(ctx.output))  # type: ignore
 
 
 class TestBlockingHandler(unittest.TestCase):
@@ -172,7 +172,7 @@ class TestBlockingHandler(unittest.TestCase):
 
         handler = unittest.mock.Mock(side_effect=save_thread)
         handler.__name__ = "handler_mock"
-        wrapped_handler = base.blocking_handler()(handler)  # type: ignore
+        wrapped_handler = base.blocking_handler()(handler)
         a.trigger(wrapped_handler)
         await a.publish(TOTALLY_RANDOM_NUMBER, [self])
         await asyncio.sleep(0.01)
